<?php

namespace Litepie\User\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\User\Events\ClientAction as ClientActionEvent;
use Litepie\User\Notifications\ClientAction as ClientActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ClientAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ClientActionEvent.
     *
     * @param   ClientActionEvent  $event
     * @return mixed
     */
    public function handle(ClientActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $client.
     *
     * @param   ClientActionEvent  $event
     * @return void
     */
    public function create(ClientActionEvent $event)
    {
        $client = $event->client->client;
        Notification::send($client, new ClientActionNotification($event));
    }

    /**
     * Handle the ClientActionEvent as a listener.
     *
     * @param   ClientActionEvent  $event
     * @return mixed
     */
    public function asListener(ClientActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ClientActionEvent  $event
     * @return bool
     */
    private function isAllowed(ClientActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
