<?php

namespace Litepie\User\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\User\Events\ClientWorkflow as ClientWorkflowEvent;
use Litepie\User\Notifications\ClientWorkflow as ClientWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ClientWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ClientWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $client is submitted.
     *
     * @param ClientWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ClientWorkflowEvent $event)
    {
        $client = $event->client->client;
        Notification::send($client, new ClientWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $client is published.
     *
     * @param ClientWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ClientWorkflowEvent $event)
    {
        $client = $event->client->client;
        Notification::send($client, new ClientWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $client is approved.
     *
     * @param ClientWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ClientWorkflowEvent $event)
    {
        $client = $event->client->client;
        Notification::send($client, new ClientWorkflowNotification($event));
    }

    /**
     * Handles the $client workflow event as a listener.
     *
     * @param ClientWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ClientWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
