<?php

namespace Litepie\User\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Litepie\User\Models\Client;

class ClientPolicy
{


    /**
     * Determine if the given user can view the client.
     *
     * @param Authenticatable $user
     * @param Client $client
     *
     * @return bool
     */
    public function view(Authenticatable $user, Client $client)
    {
        if ($authUser->canDo('user.client.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $client->is_owner;
    }

    /**
     * Determine if the given user can create a client.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('user.client.create');
    }

    /**
     * Determine if the given user can update the given client.
     *
     * @param Authenticatable $user
     * @param Client $client
     *
     * @return bool
     */
    public function update(Authenticatable $user, Client $client)
    {
        if ($user->canDo('user.client.edit') && $user->isAdmin()) {
            return true;
        }

        return $client->is_owner;
    }

    /**
     * Determine if the given user can delete the given client.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Client $client)
    {
        return $client->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
